#include "svc_basic.h"

void svc_basic_auth(thread t, jpacket p, xmlnode query)
{
    session s;
    thread tnew;
    mapi m;
    char *username, *password, *resource;

    /* set this packet as the response to the IQ */
    xmlnode_hide_attrib(p->x, "type");
    xmlnode_put_attrib(p->x,"type","result");

    username = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "username")));
    password = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "password")));
    resource = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "resource")));

    if(resource == NULL)
    {
        svc_basic_auth_result(query,"illegal","Invalid Authenticaiton Query");
        return;
    }

    /* do they want to be anonymous? */
    if(username == NULL || password == NULL)
    {
        debug("SVC_BASIC: creating anonymous resource");
        if(resource_lookup(NULL,resource) != NULL)
        {
            svc_basic_auth_result(query,"resource","Requested Resource In Use");
            return;
        }

        /* attempt to create a session for the anonymous resource */
        p->from->resource = resource;
        if(!svc_basic_new(t, p->from))
        {
            svc_basic_auth_result(query,"session","Session Creation Failed");
            return;
        }
        return;
    }

    /* create a mapi for auth */
    m = mapi_new(M_PHASE_AUTH, p->p, NULL, p);
    m->auth = M_AUTH_ERROR;
    m->auth_error = pstrdup(m->p, "Unknown User");
    m->username = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "username")));
    m->password = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "password")));
    m->resource = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "resource")));

    /* see if we please the gods */
    module_dispatch(m);
    if(m->auth == M_AUTH_OK)
    {
        if(resource_lookup(p->from->user, p->from->resource) != NULL)
        {
            svc_basic_auth_result(query,"resource","Requested Resource In Use");
            return;
        }

        /* attempt to create a session for these credentials */
        p->from->resource = resource;
        p->from->user = username;
        if(!svc_basic_new(t, p->from))
        {
            svc_basic_auth_result(query,"session","Session Creation Failed");
            return;
        }
        svc_basic_auth_result(query,"ok",NULL);
    }else{
        svc_basic_auth_result(query,"auth",m->auth_error);
    }
}


void svc_basic_auth_result(xmlnode query, char *type, char *message)
{
    xmlnode result;

    debug("SVC_BASIC: auth result %s/%s",type,message);

    xmlnode_hide(xmlnode_get_tag(query, "password"));
    xmlnode_hide(xmlnode_get_tag(query, "result"));
    result = xmlnode_insert_tag(query, "result");
    xmlnode_put_attrib(result,"type",type);
    if(message != NULL)
        xmlnode_insert_cdata(result,message,strlen(message));
}
