#include "svc_basic.h"

int svc_basic_new(thread t, jid owner)
{
    thread tnew;
    session s;

    debug("creating new session for %s",jid_full(owner));
    tnew = session_create(owner, svc_basic_onSend, (void *)t);
    if(tnew != NULL)
    {
        s = (session)(tnew->data);
        svc_basic_lookup(CMD_SET,t,s);
        return 1;
    }

    return 0;
}


struct svc_basic_connections
{
    session s;
    thread t;
    struct svc_basic_connections *next;
};

/* internal lookup routine, associating the connection thread and it's session */
session svc_basic_lookup(int cmd, thread t, session s)
{
    static struct svc_basic_connections *l = NULL;
    struct svc_basic_connections *next, *prev;
 
    if(t == NULL)
        return NULL;
 
    switch(cmd)
    {
        case CMD_SET: /* saves this thread/session pair */
            if(svc_basic_lookup(CMD_CHECK,t,NULL) != NULL)
                svc_basic_lookup(CMD_DEL,t,NULL); /* make sure we remove any old associations */

            next = malloc(sizeof(struct svc_basic_connections));
            next->s = s;
            next->t = t;
            next->next = l;
            l = next;
            return s;
            break;
        case CMD_CHECK: /* returns the session for the given thread or returns NULL */
            next = l;
            while(next != NULL)
            {
                if(t == next->t)
                    return next->s;
                next = next->next;
            }
            break;
        case CMD_DEL: /* removes this thread/session association */
            prev = next = l;
            while(next != NULL)
            {
                if(t == next->t)
                {
                    if(next == l)
                        l = next->next;
                    else
                        prev->next = next->next;
                    free(next);
                    return NULL;
                }
                prev = next;
                next = next->next;
            }
            break;
    }

    return NULL;
}


