#include "jserver.h"

/* start all system threads here, init any globals */
void config_startup(void)
{
    debug("JSERVER STARTUP!");
    pth_spawn(PTH_ATTR_DEFAULT, external_main, NULL); /* the external thread */
    pth_spawn(PTH_ATTR_DEFAULT, unknown_main, NULL); /* the thread handling all unknown resources */
    pth_spawn(PTH_ATTR_DEFAULT, server_main, NULL); /* all traffic to server resources */
    pth_spawn(PTH_ATTR_DEFAULT, remote_main, NULL); /* handles remote resources */
}

/* this is where everything is configured */
void config_handle(xmlnode x)
{
    xmlnode next, l, m;
    int listen_flag = 0;

    debug("config_handle");

    /* we got the chunk from the config file, 
    make sure we init everything depending on config reguardless if they were configured  */
    next = xmlnode_get_firstchild(x);
    l = m = NULL;
    while(next != NULL)
    {
        if(j_strcmp(xmlnode_get_name(next),"name") == 0)
            config_names(CMD_SET,xmlnode_get_data(xmlnode_get_firstchild(next)));
        if(j_strcmp(xmlnode_get_name(next),"transport") == 0)
            config_transports(CMD_SET,NULL,next);
        if(j_strcmp(xmlnode_get_name(next),"listen") == 0)
        {
            listen_flag = 1;
            config_listen(next);
        }
        if(j_strcmp(xmlnode_get_name(next),"service") == 0)
            config_service(next);
        if(j_strcmp(xmlnode_get_name(next),"log") == 0)
            l = next;
        if(j_strcmp(xmlnode_get_name(next),"modules") == 0)
            m = next;
        next = xmlnode_get_nextsibling(next);
    }

    if(listen_flag == 0)
        config_listen(NULL); /* make sure we start listening even if it wasn't configure */

    logger_init(l);
    modules_init(m);
}

struct names
{
    char *name;
    struct names *next;
};

/* this is a special internal name resolver for the dns name(s) of this transport */
char *config_names(int cmd, char *name)
{
    static struct names *n = NULL, *get = NULL;
    struct names *new;
    
    switch(cmd)
    {
        case CMD_SET: /* only intended to be run at config time */
            if(config_names(CMD_CHECK,name) != NULL || name == NULL)
                return NULL;
            etherx_alias(name);
            new = malloc(sizeof(struct names));
            new->name = j_strdup(name);
            new->next = n;
            n = new;
            break;
        case CMD_CHECK:
            new = n;
            while(new != NULL)
            {
                if(j_strcmp(name,new->name) == 0)
                    return new->name;
                new = new->next;
            }
            break;
        case CMD_GET: /* loops through name list, returns NULL at end */
            if(get == NULL)
                get = n;
            else
                get = get->next;
            if(get != NULL)
                return get->name;
            break;
    }
    return NULL;
}

struct transports
{
    char *id;
    char *name;
    struct transports *next;
};

/* 
this is a container to store the configured transports available for users
*/
char *config_transports(int cmd, char *id, xmlnode x)
{
    static struct transports *t = NULL, *get = NULL;
    struct transports *next;

    switch(cmd)
    {
        case CMD_SET: /* extract the transport n-fo from the config */
            next = malloc(sizeof(struct transports));
            next->id = j_strdup(xmlnode_get_attrib(x,"id"));
            next->name = j_strdup(xmlnode_get_data(xmlnode_get_firstchild(x)));
            next->next = t;
            t = next;
            break;
        case CMD_CHECK: /* find the given id and return the name */
            next = t;
            while(next != NULL)
            {
                if(j_strcmp(next->id, id) == 0)
                    return next->name;
                next = next->next;
            }
            break;
        case CMD_GET: /* loop through all of the id's */
            if(get == NULL)
                get = t;
            else
                get = get->next;
            if(get != NULL)
                return get->name;
            break;
    }

    return NULL;
}

void config_service(xmlnode x)
{
    /* dso load the service and initialize it */
    char *dso;

    dso = xmlnode_get_attrib(x, "dso");
    dso_load(dso, "init", (void *)x);
}

/* start the listen thread(s) based on the configuration */
void config_listen(xmlnode x)
{
    char *ip;
    int port;
 
    ip = xmlnode_get_data(xmlnode_get_firstchild(x));
    if(xmlnode_get_attrib(x, "port") == NULL || sscanf(xmlnode_get_attrib(x, "port"), "%d", &port) == 0)
        port = 5222;
 
/*    printf("listening on %s:%d\n",ip,port);*/
 
    tlisten_new(port, ip, conn_connect, NULL);
}
