#include "jserver.h"

void etherx_receive(epacket e)
{
    /* handles incoming packets from etherx */
    jpacket p;

    if(e == NULL)
    {
        config_startup();
        return;
    }

    switch(e->type)
    {
        case EPACKET_NORMAL:
            debug("EXTERNAL: got epacket from[%s]: %s",e->from, xmlnode2str(e->x));
            xmlnode_hide_attrib(e->x, "etherx:to");
            xmlnode_hide_attrib(e->x, "etherx:from");
            p = jpacket_new(xmlnode_dup(e->x));
            deliver(p);
            break;
        case EPACKET_ERROR:
            debug("EXTERNAL: got error epacket bounce that was to[%s]: %s",e->to, xmlnode2str(e->x));
            break;
        case EPACKET_REGISTRY:
            config_handle(e->x);
            break;
    }
}

void *external_main(void *arg)
{
    /* thread that waits for messages intended to be sent externally, calls etherx_write */
    /* service phase for externally bound packets? */
    /* use transports and jid->type, deliver to transport for type */
    pth_event_t ev;
    pth_msgport_t mp;
    jpq q;
    epacket e;
    
    debug("THREAD:EXTERNAL starting");

    mp = pth_msgport_create("external");
    ev = pth_event(PTH_EVENT_MSG,mp);

    while(1)
    {
        pth_wait(ev);
        while((q = (jpq)pth_msgport_get(mp)) != NULL)
        {
            if(q->p->to == NULL || q->p->from == NULL)
            {
                debug("THREAD:EXTERNAL invalid recipients for external delivery");
                break;
            }
            debug("THREAD:EXTERNAL received a message!");
            xmlnode_put_attrib(q->p->x, "to", jid_full(q->p->to));
            xmlnode_put_attrib(q->p->x, "from", jid_full(q->p->from));
            e = epacket_new(q->p->x, q->p->to->server, q->p->from->server, NS_SERVER);
            etherx_send(e);
        }
    }
    
    pth_event_free(ev,PTH_FREE_ALL);
    pth_msgport_destroy(mp);
}
