#include "jserver.h"

resource resources = NULL;

resource resource_create(int type, char *user, char *res, pth_msgport_t handler)
{
    resource new;

    debug("RESOURCE:ADD %d %s/%s %d\n",type,user,res,handler);
    new = malloc(sizeof(_resource));
    new->type = type;
    new->user = j_strdup(user);
    new->res = j_strdup(res);
    new->handler = handler;
    new->next = resources;
    resources = new;

    return new;
}

resource resource_lookup(char *user, char *res)
{
    resource next;

    if(user == NULL && res == NULL)
        return NULL;

    next = resources;
    while(next != NULL)
    {
/*        printf("RESOURCE:CHECK %d %s/%s %d\n",next->type,next->user,next->res,next->handler);*/
        if((j_strcmp(user, next->user) == 0 || user == NULL) && j_strcmp(res, next->res) == 0)
            return next;
        next = next->next;
    }
    return NULL;
}

void resource_free(resource r)
{
    resource next;

    if(r == NULL)
        return;
    
    if(resources == r)
    {
        resources = r->next;
    }else{
        next = resources;
        while(next != NULL)
        {
            if(next->next == r)
            {
                next->next = next->next->next;
                break;
            }
            next = next->next;
        }
    }

    free(r->user);
    free(r->res);
    free(r);
}

void deliver(jpacket p)
{
    resource lookup;
    static pth_msgport_t unknown_mp = NULL;
    static pth_msgport_t external_mp = NULL;
    static pth_msgport_t server_mp = NULL;

    debug("deliver()");

    if(unknown_mp == NULL)
        unknown_mp = pth_msgport_find("unknown");
    if(external_mp == NULL)
        external_mp = pth_msgport_find("external");
    if(server_mp == NULL)
        server_mp = pth_msgport_find("server");

    /* important main function, takes any packet and attempts to deliver it to the correct session/thread */
    /* calls resource_lookup, creates/sends message, etc */

    if(p->to == NULL)
    {
        warn("delivery","no recipient, bouncing");
        /* XXX should add some error stuff here and bounce
        p->to = p->from; */
        return;
    }

    /* first, is it destined externally? */
    if(config_names(CMD_CHECK,p->to->server) == NULL)
    {
        psend(external_mp,p);
        return;
    }

    /* is it for us? */
    if(p->to->user == NULL && p->to->resource == NULL && p->to->server != NULL)
    {
        psend(server_mp,p);
        return;
    }

    /* is this for a known resource? */
    lookup = resource_lookup(p->to->user, p->to->resource);

    /* unknown resources handled by unknown thread */
    if(lookup == NULL)
    {
        psend(unknown_mp,p);
        return;
    }

    psend(lookup->handler,p);
}

void psend(pth_msgport_t mp, jpacket p)
{
    jpq q;

    if(p == NULL || mp == NULL)
        return;

    q = pmalloc(p->p, sizeof(_jpq));
    q->p = p;

    pth_msgport_put(mp, (pth_message_t *)q);
}
