#include "jserver.h"

thread session_create(jid owner, session_onSend evt, void *evt_data)
{
    /* creats a new session, registers the resource for it, and spawns a thread to handle it */
    thread t;
    session s;

    if(owner == NULL || owner->resource == NULL) /* illegal */
        return NULL;
 
    debug("session_create");
 
    t = thread_new();
    s = pmalloc(t->p, sizeof(struct session_struct));
    s->mp_from = pth_msgport_create("session_from");
    s->mp_to = pth_msgport_create("session_to");
    s->id = pstrdup(t->p, jid_full(owner));
    s->r = resource_create(RESOURCE_SESSION, owner->user, owner->resource, s->mp_to);
    s->p = t->p;
    s->exit_flag = 0;
    s->send_evt = evt;
    s->send_evt_data = evt_data;
    t->data = (void *)s;
    t->id = pth_spawn(PTH_ATTR_DEFAULT, session_main, (void *)t);
    pth_yield(NULL); /* let it have a chance to start up, XXX NULL should be t->id but it seems to be broken in pth */
 
    return t;
}

void session_end(session s)
{
    jpq q;

    if(s == NULL)
        return;

    /* unlist ourselves */
    resource_free(s->r);

    /* XXX presence: if this session still thinks it's online, tell it otherwise */

    /* flag the session to exit ASAP and send it a dummy message so it gets CPU time */
    s->exit_flag = 1;
    q = pmalloc(s->p, sizeof(_jpq));
    q->p = NULL;
    pth_msgport_put(s->mp_from, (pth_message_t *)q);

}

void *session_main(void *arg)
{
    thread t = (thread)arg;
    session s = (session)(t->data);
    pth_event_t from, to, ring;
    jpq q;
    
    debug("THREAD:SESSION starting");

    from = pth_event(PTH_EVENT_MSG,s->mp_from);
    to = pth_event(PTH_EVENT_MSG,s->mp_to);
    ring = pth_event_concat(from, to, NULL);

    module_dispatch(mapi_new(M_PHASE_SESSION_ON,t->p,s,NULL));

    while(1)
    {
        pth_wait(ring);

        while(1)
        {
            /* this loop looks for messages from our user that need to be delivered elsewhere */
            q = (jpq)pth_msgport_get(s->mp_from);
            if(q == NULL || q->p == NULL)
                break;

            debug("THREAD:SESSION:FROM received a message!");

            if(s->r->user != NULL)
            {
                switch(q->p->type)
                {
                    case JPACKET_MESSAGE:
                        module_dispatch(mapi_new(M_PHASE_MESSAGE_OUT,q->p->p,s,q->p));
                        break;
                    case JPACKET_PRESENCE:
                        module_dispatch(mapi_new(M_PHASE_PRESENCE_OUT,q->p->p,s,q->p));
                        break;
                }
            }

            /* pass these to the general delivery function */
            deliver(q->p);
        }

        while(1)
        {
            /* this loop looks for messages to our user that need to be sent to them */
            q = (jpq)pth_msgport_get(s->mp_to);
            if(q == NULL || q->p == NULL)
                break;

            /* XXX if(s->exit_flag) we're exiting and should bounce these suckers! */

            debug("THREAD:SESSION:TO received a message!");

            if(s->r->user != NULL)
            {
                switch(q->p->type)
                {
                    case JPACKET_MESSAGE:
                        module_dispatch(mapi_new(M_PHASE_MESSAGE_IN,q->p->p,s,q->p));
                        break;
                    case JPACKET_PRESENCE:
                        module_dispatch(mapi_new(M_PHASE_PRESENCE_IN,q->p->p,s,q->p));
                        break;
                    case JPACKET_IQ:
                        module_dispatch(mapi_new(M_PHASE_IQ,q->p->p,s,q->p));
                        break;
                }
            }

            /* deliver outgoing for this session to the onSend event passed when this session was created */
            (s->send_evt)(t, q->p);
        }

        if(s->exit_flag)
            break;
    }

    debug("THREAD:SESSION exiting");

    module_dispatch(mapi_new(M_PHASE_SESSION_OFF,t->p,s,NULL));

    pth_event_free(ring,PTH_FREE_ALL);
    pth_msgport_destroy(s->mp_from);
    pth_msgport_destroy(s->mp_to);
}

